// -*- C++ -*-
//
// dlib.cpp - DragonGem main utility library
//
// Written by: Michael Westra
// Written for: CIS 587, Summer 2000 at UofMD
//----------------------------------------------------------------------

#define DLIB_EXTERNS
#include "dlib.h"

// Several internal routines...
//**********************************************************************

// Internal helper function to convert from block type (bitmap) to animation value.
//----------------------------------------------------------------------
int Piece2Anim(int piece, int reverse) {
  int offset = 0;
  if (reverse)
    offset = 10;

  switch (piece) {
  case RED_BLOCK:    return (0+offset);
  case BLUE_BLOCK:   return (1+offset);
  case PURPLE_BLOCK: return (2+offset);
  case ORANGE_BLOCK: return (3+offset);
  case GREEN_BLOCK:  return (4+offset);
  case CYAN_BLOCK:   return (5+offset);
  case YELLOW_BLOCK: return (6+offset);
  case SKULL_BLOCK:  return (SKULL_ANIM+offset);
  case ANY_BLOCK:    return (ANY_ANIM+offset);
  case BOMB_BLOCK:   return (BOMB_ANIM+offset);
    
  case EMPTY_BLOCK:
  default:
    return 0;
  }
}
//**********************************************************************

// Function to initialize game board...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Init_Board() {
  int i,j;

  curr_level = 1;
  curr_score = 0;

  curr_block.type_list[0] = curr_block.type_list[1] = EMPTY_BLOCK;
  next_block.type_list[0] = next_block.type_list[1] = EMPTY_BLOCK;
  curr_block.row = BOARD_HEIGHT;
  curr_block.col = 3;

  for (i=0; i<BLOCK_SIZE; i++) {
    curr_block.data_list[i] = &curr_block._real_data[i];
    next_block.data_list[i] = &next_block._real_data[i];

    if (curr_block.valid_bob)
      Destroy_BOB( curr_block.data_list[i] );
    if (next_block.valid_bob)
      Destroy_BOB( next_block.data_list[i] );

    int x,y;
    MAP_BLOCK(curr_block.row+i, curr_block.col, x,y);

    if (i) {
      // There can/should be only Bob, we can clone at will!
      Clone_BOB(curr_block.data_list[0], curr_block.data_list[i]);
    } else {
      int idx;

      Load_Bitmap_File(&bitmap8bit, "BLOCKS.BMP");

      if (! Create_BOB(curr_block.data_list[0], // BOB struct
                       x,y,    // x/y pos
                       30, 30, // size
                       NUM_TOT_FRAMES, // number of frames
                       // multiple animation possibilities
                       BOB_ATTR_MULTI_ANIM | BOB_ATTR_ANIM_ONE_SHOT | BOB_ATTR_VISIBLE, 
                       0       // place in vram
                       ) ) {
#ifdef DEBUG
        Write_Error("\nCould not create BOB!!\n");
#endif
        return;
      }

      // grab out block info
      for (idx = 0; idx < 7; idx++) {
        Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit, idx,
                       idx, 0, BITMAP_EXTRACT_MODE_CELL);

        // Load derived frames
        for (j = 0; j < ANIM_LEN; j++) {
#ifdef DEBUG
          Write_Error("Loading anim_frame %d [%d][%d]\n", idx*ANIM_LEN+ANIM_OFFSET+j, j, idx);
#endif
          Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit,
                         idx*ANIM_LEN+ANIM_OFFSET+j,
                         idx, j, BITMAP_EXTRACT_MODE_CELL);
        }
      }

      // load skull
      Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit, SKULL_ANIM,
                     7, 0, BITMAP_EXTRACT_MODE_CELL);
      // Load derived frames
      for (j = 0; j < ANIM_LEN; j++) {
#ifdef DEBUG
        Write_Error("Loading anim_frame %d [%d][%d]\n", SKULL_ANIM*ANIM_LEN+ANIM_OFFSET+j, j, SKULL_ANIM);
#endif
        Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit,
                       SKULL_ANIM*ANIM_LEN+ANIM_OFFSET+j,
                       7, j, BITMAP_EXTRACT_MODE_CELL);
      }

      // load rainbow
      Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit, ANY_ANIM,
                     9, 0, BITMAP_EXTRACT_MODE_CELL);
      // Load derived frames
      for (j = 0; j < ANIM_LEN; j++) {
#ifdef DEUBG
        Write_Error("Loading anim_frame %d [%d][%d]\n", ANY_ANIM*ANIM_LEN+ANIM_OFFSET+j, j, ANY_ANIM);
#endif
        Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit,
                       ANY_ANIM*ANIM_LEN+ANIM_OFFSET+j,
                       9, j, BITMAP_EXTRACT_MODE_CELL);
      }

      // load spotty bomb
      Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit, BOMB_ANIM,
                     11, 0, BITMAP_EXTRACT_MODE_CELL);
      // Load derived frames
      for (j = 0; j < ANIM_LEN; j++) {
#ifdef DEBUG
        Write_Error("Loading anim_frame %d [%d][%d]\n", BOMB_ANIM*ANIM_LEN+ANIM_OFFSET+j, j, BOMB_ANIM);
#endif
        Load_Frame_BOB(curr_block.data_list[0], &bitmap8bit,
                       BOMB_ANIM*ANIM_LEN+ANIM_OFFSET+j,
                       11, j, BITMAP_EXTRACT_MODE_CELL);
      }

      // setup bob's master anim sequences
      for (idx = 0; idx <= BOMB_ANIM; idx++) {
        int forward_seq[10];
        //int backward_seq[10];

        // fill forward sequence
        for (j=0; j<ANIM_LEN; j++) {
          forward_seq[j] = idx*ANIM_LEN+ANIM_OFFSET+j;  // calculate frame value
          //backward_seq[j] = idx*ANIM_LEN+ANIM_OFFSET+(ANIM_LEN-1)-j;
#ifdef DEBUG
          Write_Error("Loading anim_seq #%d %d [%d]\n", idx, j, forward_seq[j]);
          //Write_Error("Loading anim_seq #%d %d [%d]\n", idx+10, j, backward_seq[j]);
#endif
        }
        Load_Animation_BOB(curr_block.data_list[0],
                           idx, ANIM_LEN, forward_seq);
        /* back anim sequence not used currently
             (tried this for blocks "fading out" for the dropping animation, but
             did "feel right" because it was too visually distracting.) */
        //Load_Animation_BOB(curr_block.data_list[0],
        //idx+10, ANIM_LEN, backward_seq);
      }

      // unload bitmap image
      Unload_Bitmap_File(&bitmap8bit);

    } // end if first bob

    // clone next block data
    Clone_BOB(curr_block.data_list[0], next_block.data_list[i]);
    // fix position of next block display
    Set_Pos_BOB(next_block.data_list[i], 300, 400+i*BLOCK_HEIGHT);
  } // end for each bob
  curr_block.valid_bob = 1;
  next_block.valid_bob = 1;

  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      game_matrix[i][j] = EMPTY_BLOCK;
      return_matrix[i][j] = 0;

      // create positional bob and position it
      int x,y;

      Clone_BOB(curr_block.data_list[0], &bob_matrix[i][j]);
      MAP_BLOCK(i,j, x,y);
      Set_Pos_BOB(&bob_matrix[i][j], x,y);
      Set_Vel_BOB(&bob_matrix[i][j], 0, 0);  // never move!
      Set_Anim_Speed_BOB(&bob_matrix[i][j], ANIM_SPEED); // speed of animation
      // bob_matrix[i][j].curr_frame = rand()%10;
    }
  }

  // load pause image
  Load_Bitmap_File(&bitmap8bit, "PAUSE.BMP");
  Create_Bitmap(&pause_bmp,0,0,640,480);
  Load_Image_Bitmap(&pause_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
  Unload_Bitmap_File(&bitmap8bit);

  // load pause image
  Load_Bitmap_File(&bitmap8bit, "OVER.BMP");
  Create_Bitmap(&over_bmp,0,0,640,480);
  Load_Image_Bitmap(&over_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
  Unload_Bitmap_File(&bitmap8bit);

  // load background sounds
  sound_drop_id = DSound_Load_WAV("DROP.WAV");
  sound_flip_id = DSound_Load_WAV("FLIP.WAV");
  sound_remove_id = DSound_Load_WAV("REMOVE.WAV");
  sound_over_id = DSound_Load_WAV("GAMEOVER.WAV");
  sound_level_id = DSound_Load_WAV("LEVEL.WAV");

  /*
  //debug
game_matrix[0][0] = 512;
game_matrix[0][1] = 512;
game_matrix[0][2] = 512;
game_matrix[0][3] = 512;
game_matrix[0][4] = 512;
game_matrix[0][5] = 512;
game_matrix[0][6] = 512;

bob_matrix[0][0].curr_frame = 100;
bob_matrix[0][1].curr_frame = 101;
bob_matrix[0][2].curr_frame = 102;
bob_matrix[0][3].curr_frame = 103;
bob_matrix[0][4].curr_frame = 104;
bob_matrix[0][5].curr_frame = 105;
bob_matrix[0][6].curr_frame = 106;
  */
  curr_state = GAME_BLOCK_FALLING;
  Choose_Next_Block();
  Choose_Next_Block();
}

// Function to reset board after game loss...
//
// Uses:
//  everything
//---------------------------------------------------
void Reset_Board() {
  int i,j;

  curr_level = 1;
  curr_score = 0;

  curr_block.type_list[0] = curr_block.type_list[1] = EMPTY_BLOCK;
  next_block.type_list[0] = next_block.type_list[1] = EMPTY_BLOCK;
  curr_block.row = BOARD_HEIGHT;
  curr_block.col = 3;

  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      game_matrix[i][j] = EMPTY_BLOCK;
      return_matrix[i][j] = 0;
    }
  }
  curr_state = GAME_BLOCK_FALLING;
  Choose_Next_Block();
}

// Function to de-initialize game board...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Destroy_Board() {
  if (curr_block.valid_bob) {
    for (int i=BLOCK_SIZE-1; i>=0; i--) {
      Destroy_BOB(curr_block.data_list[i]);
      Destroy_BOB(next_block.data_list[i]);
    }
  }
  curr_block.valid_bob = 0;
  next_block.valid_bob = 0;

  DSound_Delete_Sound(sound_drop_id);
  DSound_Delete_Sound(sound_flip_id);
  DSound_Delete_Sound(sound_remove_id);
  DSound_Delete_Sound(sound_over_id);
  DSound_Delete_Sound(sound_level_id);
}

// Function to draw board and current block...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Draw_Board() {
  int i,j;

  // perform animation if requested
  //--------------------
  if (curr_state == GAME_ANIMATE) {
    // perform animation
    for (i=0; i<num_anim; i++) {
      if (curr_anim[i] != NULL)
        Animate_BOB(curr_anim[i]);
    }
    // check if time to terminate animation
    if (curr_anim[0]->anim_state == BOB_STATE_ANIM_DONE) {
      curr_state = next_state;
      num_anim = 0;
    }
  }

  // draw game blocks
  //--------------------
  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      if (game_matrix[i][j] != EMPTY_BLOCK)
        Draw_BOB(&bob_matrix[i][j], lpddsback);
    }
  }

  // draw falling blocks
  //--------------------
  if (curr_state != GAME_DEAD) {
    for (i=0; i<BLOCK_SIZE; i++) {
      
      // draw the block
      if ( (curr_state != GAME_PAUSED) && (curr_state != GAME_ANIMATE) )
        Move_BOB(curr_block.data_list[i]);
      Draw_BOB(curr_block.data_list[i], lpddsback);
      Draw_BOB(next_block.data_list[i], lpddsback);
    }
  }

  // check external states
  //--------------------
  if (curr_state == GAME_PAUSED) {
    // lock back buffer and copy background into it
    DDraw_Lock_Back_Surface();
    
    // draw background
    Draw_Bitmap(&pause_bmp, back_buffer, back_lpitch,1);
    
    // unlock back surface
    DDraw_Unlock_Back_Surface();
  } else if (curr_state == GAME_DEAD) {
    // lock back buffer and copy background into it
    DDraw_Lock_Back_Surface();
    
    // draw background
    Draw_Bitmap(&over_bmp, back_buffer, back_lpitch,1);
    
    // unlock back surface
    DDraw_Unlock_Back_Surface();
  }

}

// Function to populate curr_block with randomly selected block (based on level)
//
// Uses:
//  curr_block (populates), curr_level
//--------------------------------------------------
void Choose_Next_Block() {
  int type, i;

  for (i=0; i<BLOCK_SIZE; i++) {
    curr_block.type_list[i] = next_block.type_list[i];
    curr_block.data_list[i]->curr_frame = next_block.data_list[i]->curr_frame;

    type = rand()%2000;

    if (type < 100) {
      // 1 in 20 chance of getting 'special' block
      next_block.type_list[i] = EMPTY_BLOCK;
      
      if ( (type - curr_level) < 30 ) {
        // skull (chances increase w/ additional levels)
      next_block.type_list[i] = SKULL_BLOCK;
      next_block.data_list[i]->curr_frame = SKULL_ANIM;
      } else if (type%2 == 0) {
        // otherwise even chance of rainbow/bomb
        // rainbow
        next_block.type_list[i] = ANY_BLOCK;
        next_block.data_list[i]->curr_frame = ANY_ANIM;
      } else {
        // bomb
        next_block.type_list[i] = BOMB_BLOCK;
        next_block.data_list[i]->curr_frame = BOMB_ANIM;
      }
    } else {
      int r = rand()%7;

      next_block.type_list[i] = ( 1 << r );
      next_block.data_list[i]->curr_frame = r;
#ifdef DEBUG
      Write_Error("In Choose_Next_Block color:%d\n", r);
#endif
    } // end if special block

    // finally, set initial position and speed of block
    curr_block.row = BOARD_HEIGHT;
    curr_block.col = 3;

    int x,y;
    MAP_BLOCK(curr_block.row-i, curr_block.col, x,y);

    int new_speed = (int)(curr_level*.5+.5);

    Set_Pos_BOB(curr_block.data_list[i], x,y);
    Set_Vel_BOB(curr_block.data_list[i], 0, new_speed);
#ifdef DEBUG
    Write_Error("In Choose_Next_Block r/c:%d/%d\n", curr_block.row, curr_block.col);
    Write_Error("In Choose_Next_Block x/y:%d/%d\n", x,y);
    Write_Error("In Choose_Next_Block dy:%d\n", new_speed);
#endif

  } // end foreach block in stack
}

// Function to drop/move current to its final place.
//
// Uses:
//  curr_block
//--------------------------------------------------
void Drop_Curr_Block() {
  int curr_row = curr_block.row;
  int i;

  // play drop sound
  DSound_Play(sound_drop_id);

  curr_block.row = 1;
  curr_state = GAME_BLOCK_DOWN;

  for (i=BOARD_HEIGHT-1; i>=0; i--) {
#ifdef DEBUG
    Write_Error("Dropping at %d=%d\n", i, game_matrix[i][curr_block.col]);
#endif
    if (game_matrix[i][curr_block.col] != EMPTY_BLOCK) {
      curr_block.row = i+2;
#ifdef DEBUG
      Write_Error("Dropping into position %d\n", curr_block.row);
#endif
      return;
    }
  }
#ifdef DEBUG
  Write_Error("Dropping into position %d\n", curr_block.row);
#endif
}

// Function to figure out which blocks can be removed (3-in-a-row)
//
// Uses:
//  game_matrix, return_matrix (populates)
//--------------------------------------------------
int Find_Remove_Blocks() {
  int i,j;

#ifdef DEBUG
  Write_Error("\nIn Find_Remove_Blocks()...\n");
  Write_Error("Cleaning return matrix...\n");
#endif

  // clear return matrix
  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      return_matrix[i][j] = 0;
    }
  }

  // find 3-in-a-row situations (if any)
  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      int curr = game_matrix[i][j];

      // no point in checking if empty
      if (game_matrix[i][j] == EMPTY_BLOCK)
        continue;

      // handle special case with bomb
      if (game_matrix[i][j] == BOMB_BLOCK) {
        return_matrix[i][j] = 1;

        if (j>0) {
          // upper left
          if ( (i<BOARD_HEIGHT-1) &&
               (game_matrix[i+1][j-1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i+1][j-1] = 1;
          // left
          if ( (game_matrix[i][j-1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i][j-1] = 1;
          // lower left
          if ( (i>0) && 
               (game_matrix[i-1][j-1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i-1][j-1] = 1;
        }
        // upper
        if ( (i<BOARD_HEIGHT-1) &&
             (game_matrix[i+1][j] != EMPTY_BLOCK) &&
             (rand()%2) )
          return_matrix[i+1][j] = 1;
        // lower
        if ( (i>0) &&
             (game_matrix[i-1][j] != EMPTY_BLOCK) &&
             (rand()%2) )
          return_matrix[i-1][j] = 1;
        if (j<BOARD_WIDTH-1) {
          // upper right
          if ( (i<BOARD_HEIGHT-1) &&
               (game_matrix[i+1][j+1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i+1][j+1] = 1;
          // right
          if ( (game_matrix[i][j+1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i][j+1] = 1;
          // lower right
          if ( (i>0) && 
               (game_matrix[i-1][j+1] != EMPTY_BLOCK) &&
               (rand()%2) )
            return_matrix[i-1][j+1] = 1;
        }
      } // end if bomb block

      // horizontal
      if ( (j>0) && (j<BOARD_WIDTH-1) &&
           (game_matrix[i][j-1] & curr) &&
           (game_matrix[i][j-1] & game_matrix[i][j+1]) &&
           (game_matrix[i][j+1] & curr) )
        return_matrix[i][j-1] = return_matrix[i][j] = return_matrix[i][j+1] = 1;
      // vertical
      if ( (i>0) && (i<BOARD_HEIGHT-1) &&
           (game_matrix[i-1][j] & curr) &&
           (game_matrix[i-1][j] & game_matrix[i+1][j]) &&
           (game_matrix[i+1][j] & curr) )
        return_matrix[i-1][j] = return_matrix[i][j] = return_matrix[i+1][j] = 1;
      if ( (j>0) && (j<BOARD_WIDTH-1) && (i>0) && (i<BOARD_HEIGHT-1) ) {
        // upper diagonal
        if ( (game_matrix[i-1][j-1] & curr) &&
             (game_matrix[i-1][j-1] & game_matrix[i+1][j+1]) &&
             (game_matrix[i+1][j+1] & curr) )
          return_matrix[i-1][j-1] = return_matrix[i][j] = return_matrix[i+1][j+1] = 1;
        // lower diagonal 
        if ( (game_matrix[i-1][j+1] & curr) &&
             (game_matrix[i-1][j+1] & game_matrix[i+1][j-1]) &&
             (game_matrix[i+1][j-1] & curr) )
          return_matrix[i-1][j+1] = return_matrix[i][j] = return_matrix[i+1][j-1] = 1;
      }
    } // end for j
  } // end for i

  // finally count number of filled positions
  int count = 0;
  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
#ifdef DEBUG
      //      Write_Error("Checking [%d][%d]=%d...%d\n",i,j,game_matrix[i][j],return_matrix[i][j]);
      Write_Error("%d\t",game_matrix[i][j]);
#endif
      if (return_matrix[i][j])
        count++;
    }
#ifdef DEBUG
    Write_Error("\n");
#endif

  }
#ifdef DEBUG
  Write_Error("Found %d blocks in rows...\n",count);
#endif
  return(count);
}

// Function to setup animation sequence for removed blocks
//
// Uses:
//  bob_matrix, game_matrix, return_matrix, curr_anim/num_anim (set)
//----------------------------------------------------------------------
void Init_Anim_Remove_Blocks() {
  int i,j;

  for (i=0; i<BOARD_HEIGHT; i++) {
    for (j=0; j<BOARD_WIDTH; j++) {
      if (return_matrix[i][j]) {
        // determine animation sequence to play
        curr_anim[num_anim] = &bob_matrix[i][j];
        Set_Animation_BOB(curr_anim[num_anim], Piece2Anim(game_matrix[i][j], 0));
        curr_anim[num_anim]->anim_state = 0;

#ifdef DEBUG
        Write_Error("Setting animation: #%d type:%d anim:%d\n", num_anim, game_matrix[i][j], Piece2Anim(game_matrix[i][j], 0));
#endif
        num_anim++;
      } // end if removal position
    } // end foreach column
  } // end foreach row
}

// Function to drop blocks by removing marked blocks.
//
// Uses:
//  game_matrix (updated), return_matrix
//--------------------------------------------------
void Drop_Remove_Blocks() {
  int i, j, tmp_val, empty_pos, filled_pos;

#ifdef DEBUG
  Write_Error("\nIn Drop_Remove_Blocks()...\n");
#endif

  for (j=0; j<BOARD_WIDTH; j++) {
    // empty out positions where match found
    for (i=0; i<BOARD_HEIGHT; i++) {
      if (return_matrix[i][j])
        game_matrix[i][j] = EMPTY_BLOCK;
    }

    // drop down to fill-in empty blocks
    empty_pos = filled_pos = 0;
    // This is almost the most *strange* thing I've ever seen.
    // This line is syntactically correct, but when the dumb 'ol MS compiler
    //  sees it, it complains:
    //    c:\TMP\dragon\dlib.cpp(301) : fatal error C1001: INTERNAL COMPILER ERROR
    //    (compiler file 'E:\8168\vc98\p2\src\P2\main.c', line 494)
    //    Please choose the Technical Support command on the Visual C++
    //    Help menu, or open the Technical Support help file for more information
	//
	// After consulting the man pages, the compiler gets bound up with some
	//  constructs and optimization flags.  Microsoft's "solution" is to recode
	//  around their crappy compiler, or turn off optimizations.  One could have
	//  gotten "that answer" and a less expensive product with g++!!!
	// Our solution is to compile w/o speed optimizations...
    //----------------------------------------------------------------------
    while ( (empty_pos < BOARD_HEIGHT) && (filled_pos < BOARD_HEIGHT) ) {

      //#ifdef DEBUG
      //Write_Error("searching empty:%d, fill:%d\n", empty_pos, filled_pos);
      //#endif

      // find next full position
      if (game_matrix[filled_pos][j] == EMPTY_BLOCK) {
        filled_pos++;
        continue;
      }

      // find next empty position
      if (game_matrix[empty_pos][j] != EMPTY_BLOCK) {
        empty_pos++;
        continue;
      }

      // empty must always trail full
      if (empty_pos > filled_pos) {
      //if (empty_pos < filled_pos) {
        filled_pos = empty_pos + 1;
        continue;
      }

      // otherwise swap the values and advance pointers
      /*
        #ifdef DEBUG
        Write_Error("swapping [%d][%d]%d and [%d][%d]%d\n", 
        filled_pos, j,
        game_matrix[filled_pos][j],
        empty_pos, j,
        game_matrix[empty_pos][j]);
        #endif
      */

      tmp_val = game_matrix[filled_pos][j];
      game_matrix[filled_pos][j] = game_matrix[empty_pos][j];
      game_matrix[empty_pos][j] = tmp_val;

      tmp_val = bob_matrix[filled_pos][j].curr_frame;
      bob_matrix[filled_pos][j].curr_frame = bob_matrix[empty_pos][j].curr_frame;
      bob_matrix[empty_pos][j].curr_frame = tmp_val;

      filled_pos++;
      empty_pos++;
                         
    } // while potential blocks left to swap
  } // end for j (columns)
}

// Function to determine how many points to award
//
// Paramaters:
//  num_blocks - number of blocks removed in this cycle
//  return     - returns # points to add
//--------------------------------------------------
int Calculate_Points(int num_blocks) {
  return (num_blocks * num_blocks * 10);
}

// Function to determine if current block has 
//  collided with bottom/another block
//
// Uses:
//  game_matrix, curr_block
//  return - true iff should stop falling
//----------------------------------------------------------------------
bool Check_Down_Curr_Block() {
  int c;
  UNMAP_BLOCK(curr_block.data_list[0]->x, curr_block.data_list[0]->y, curr_block.row, c);
  if (game_matrix[curr_block.row-BLOCK_SIZE][curr_block.col] != EMPTY_BLOCK) {
    // hit another block

    // Check that we're not in a "loose" situtation where
    // the current block is still at the 'top' of the screen.
    if (curr_block.row == BOARD_HEIGHT) {
      // we didn't hit anything, we're dead...
      curr_state = GAME_DEAD;

      // play death sound
      DSound_Play(sound_over_id);

      return false;
    }

    return true;
  } else if (curr_block.row < BLOCK_SIZE) {
    // hit bottom of game board
    return true;
  } else {
    return false;
  }
}

