// -*- C++ -*-
//
// dlib.h - DragonGem main utility library
//
// Written by: Michael Westra
// Written for: CIS 587, Summer 2000 at UofMD
//----------------------------------------------------------------------

#ifndef __DLIB_H__
#define __DLIB_H__

// output detailed debugging dump file...
//#define DEBUG

// INCLUDES
//----------------------------------------------------------------------


#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>  // directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>
#include "T3DLIB1.h" // game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

// DEFINES
//----------------------------------------------------------------------

// define block types
#define EMPTY_BLOCK    0
#define RED_BLOCK      1
#define BLUE_BLOCK     2
#define PURPLE_BLOCK   4
#define ORANGE_BLOCK   8
#define GREEN_BLOCK    16
#define CYAN_BLOCK     32
#define YELLOW_BLOCK   64
#define SKULL_BLOCK    128
#define ANY_BLOCK      255
#define BOMB_BLOCK     512

// define cooresponding animation index
#define SKULL_ANIM     7
#define ANY_ANIM       8
#define BOMB_ANIM      9

// define game states
#define GAME_PAUSED         0 // game is paused
#define GAME_BLOCK_FALLING  1 // block is falling (accept input)
#define GAME_ANIMATE        2 // block is in flipping animation state
#define GAME_BLOCK_DOWN     3 // block has just hit bottom
#define GAME_CHECK_MATRIX   4 // block has fallen, check for block removal
#define GAME_DEAD           5 // game over

#define BOARD_WIDTH         7 // board is x wide
#define BOARD_HEIGHT       15 // board is x high

#define BLOCK_SIZE          2 // blocks contain 1 or 2 elements
#define BLOCK_WIDTH        30 // block is 30 pixels wide
#define BLOCK_HEIGHT       30 // block is 30 pixels high
#define BOARD_XOFF         46 // board is offset x pixels
#define BOARD_YOFF        -28 // board is offset y pixels

#define NUM_TOT_FRAMES    110 // number of animation frames used totally
#define NUM_FILE_FRAMES    10 // number of animation frames in loaded file
#define ANIM_LEN           10 // number of frames in animation
#define ANIM_OFFSET        10 // start of animation sequences
#define ANIM_SPEED          3 // speed of animation sequence
#define ANIM_FRAMES (ANIM_SPEED+1)*ANIM_LEN // number of frames animation lasts for
#define ANIM_MAX           16 // max number of concurrent animations

// DATA STRUCTURES
//----------------------------------------------------------------------
typedef struct _struct_falling_block {
  int type_list[BLOCK_SIZE];  // type of block for block
  int row, col;               // current row/col of upper left corner of block
  int valid_bob;              // Is BOB currently valid???
  BOB *data_list[BLOCK_SIZE]; // virtual physical representation of block
  BOB _real_data[BLOCK_SIZE]; // (storage) of block
} Falling_Block, *Falling_Block_Ptr;


#ifdef DLIB_EXTERNS
#define EXTERNS /**/
#else
#define EXTERNS extern /**/
#endif

// GLOBALS for game
//   (I feel "dirty" doing this (using globals - almost like using goto's),
//    but it's the style for the LaMothe book...)
EXTERNS int game_matrix[BOARD_HEIGHT][BOARD_WIDTH]; // main game board representation
EXTERNS BOB bob_matrix[BOARD_HEIGHT][BOARD_WIDTH];  // matrix of animation sprites
EXTERNS int return_matrix[BOARD_HEIGHT][BOARD_WIDTH]; // matrix to flag found blocks to remove
EXTERNS Falling_Block next_block; // next block
EXTERNS Falling_Block curr_block; // currently 'falling' block

EXTERNS int curr_level;
EXTERNS int curr_score;
EXTERNS int curr_state;
EXTERNS int next_state;  // state to have after animation sequence

EXTERNS BITMAP_IMAGE pause_bmp; // game paused
EXTERNS BITMAP_IMAGE over_bmp;  // game over

EXTERNS BOB* curr_anim[ANIM_MAX]; // list of BOBs to animate in current frame
EXTERNS int num_anim;        // number of animations playing

EXTERNS int sound_drop_id;   // block dropping sound
EXTERNS int sound_flip_id;   // block flipping sound
EXTERNS int sound_remove_id; // block removal sound
EXTERNS int sound_over_id;   // game over sound
EXTERNS int sound_level_id;  // level up id

// MACROS
//----------------------------------------------------------------------

inline void MAP_BLOCK(int row, int col, int& x, int& y) {
  x = BLOCK_WIDTH * col + BOARD_XOFF;
  y = BLOCK_HEIGHT * (BOARD_HEIGHT-row) + BOARD_YOFF;
}

inline void UNMAP_BLOCK(int x, int y, int& row, int& col) {
  row = BOARD_HEIGHT - (y - BOARD_YOFF) / BLOCK_HEIGHT;
  col = (x - BOARD_XOFF) / BLOCK_WIDTH;
}

inline void ColPos_Curr_Block(int col) {
  // Position falling block
  int i,x,y;

  for (i=0; i<BLOCK_SIZE; i++) {
    MAP_BLOCK(0, col, x, y);
    curr_block.data_list[i]->x = x;
  }
}

// FUNCTION PROTYPES
//----------------------------------------------------------------------

// Function to initialize game board...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Init_Board();

// Function to reset board after game loss...
//
// Uses:
//  everything
//---------------------------------------------------
void Reset_Board();

// Function to de-initialize game board...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Destroy_Board();

// Function to draw board and current block...
//
// Uses:
//  everything :-)
//--------------------------------------------------
void Draw_Board();

// Function to populate curr_block with randomly selected block (based on level)
//
// Uses:
//  curr_block (populates), curr_level
//--------------------------------------------------
void Choose_Next_Block();

// Function to determine if current block has 
//  collided with bottom/another block
//
// Uses:
//  game_matrix, curr_block
//  return - true iff should stop falling
//----------------------------------------------------------------------
bool Check_Down_Curr_Block();

// Function to drop/move current to its final place.
//
// Uses:
//  curr_block
//--------------------------------------------------
void Drop_Curr_Block();

// Function to figure out which blocks can be removed (3-in-a-row)
//
// Uses:
//  game_matrix, return_matrix (populates)
// Parameters:
//  return - returns number of blocks found
//--------------------------------------------------
int Find_Remove_Blocks();

// Function to setup animation sequence for removed blocks
//
// Uses:
//  bob_matrix, game_matrix, return_matrix, curr_anim/num_anim (set)
//--------------------------------------------------
void Init_Anim_Remove_Blocks();

// Function to drop blocks by removing marked blocks.
//
// Uses:
//  game_matrix (updated), return_matrix
//--------------------------------------------------
void Drop_Remove_Blocks();

// Function to determine how many points to award
//
// Paramaters:
//  num_blocks - number of blocks removed in this cycle
//  return     - returns # points to add
//--------------------------------------------------
int Calculate_Points(int num_blocks);


#endif
