// -*- C++ -*-
//
// DRAGON.CPP - DragonGem game
//
// Written by: Michael Westra
// Written for: CIS 587, Summer 2000 at UofMD
//----------------------------------------------------------------------

// INCLUDES
//----------------------------------------------------------------------

#define INITGUID

#define WIN32_LEAN_AND_MEAN  

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>  // directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>
#include "T3DLIB1.h" // game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"
#include "dlib.h"

// DEFINES
//----------------------------------------------------------------------

// defines for windows 
#define WINDOW_CLASS_NAME "WINXCLASS"  // class name

#define WINDOW_WIDTH    320            // size of window
#define WINDOW_HEIGHT   240

#define KEYDN_CONST     0x80
#define DIKEYDOWN(data,n) (data[n] & KEYDN_CONST) // is key down?

// PROTOTYPES
//----------------------------------------------------------------------

// game console
int Game_Init(void *parms=NULL);
int Game_Shutdown(void *parms=NULL);
int Game_Main(void *parms=NULL);

// GLOBALS
//----------------------------------------------------------------------

HWND main_window_handle   = NULL; // save the window handle
HINSTANCE main_instance   = NULL; // save the instance
char buffer[80];                          // used to print text

BITMAP_IMAGE background_bmp;   // holds the background

// globals to determine if key has 'cycled'
UCHAR next_up = KEYDN_CONST,
  next_down   = KEYDN_CONST, 
  next_right  = KEYDN_CONST,
  next_left   = KEYDN_CONST,
  next_space  = KEYDN_CONST,
  next_p      = KEYDN_CONST;

// FUNCTIONS
//----------------------------------------------------------------------

//----------------------------------------------------------------------
LRESULT CALLBACK WindowProc(HWND hwnd, 
                            UINT msg, 
                            WPARAM wparam, 
                            LPARAM lparam) {
  // this is the main message handler of the system
  PAINTSTRUCT   ps;  // used in WM_PAINT
  HDC           hdc; // handle to a device context
  
  // what is the message 
  switch(msg) {	
  case WM_CREATE: {
    // do initialization stuff here
    return(0);
  } break;
  
  case WM_PAINT: {
    // start painting
    hdc = BeginPaint(hwnd,&ps);
    
    // end painting
    EndPaint(hwnd,&ps);
    return(0);
  } break;
  
  case WM_DESTROY: {
    // kill the application			
    PostQuitMessage(0);
    return(0);
  } break;
  
  default:break;
    
  } // end switch
  
  // process any messages that we didn't take care of 
  return (DefWindowProc(hwnd, msg, wparam, lparam));
  
} // end WinProc

// WINMAIN
//----------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hinstance,
                   HINSTANCE hprevinstance,
                   LPSTR lpcmdline,
                   int ncmdshow) {
  // this is the winmain function

  WNDCLASS winclass; // this will hold the class we create
  HWND     hwnd;     // generic window handle
  MSG      msg;      // generic message
  //HDC      hdc;      // generic dc
  //PAINTSTRUCT ps;    // generic paintstruct

  // first fill in the window class stucture
  winclass.style = CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
  winclass.lpfnWndProc = WindowProc;
  winclass.cbClsExtra = 0;
  winclass.cbWndExtra = 0;
  winclass.hInstance = hinstance;
  winclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  winclass.hCursor = LoadCursor(NULL, IDC_ARROW);
  winclass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
  winclass.lpszMenuName = NULL; 
  winclass.lpszClassName = WINDOW_CLASS_NAME;

  // register the window class
  if (!RegisterClass(&winclass))
    return(0);

  // create the window, note the use of WS_POPUP
  if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME, // class
                            "DragonGem", // title
                            WS_POPUP | WS_VISIBLE,
                            0,0, // x,y
                            WINDOW_WIDTH,  // width
                            WINDOW_HEIGHT, // height
                            NULL, // handle to parent 
                            NULL, // handle to menu
                            hinstance, // instance
                            NULL))) // creation parms
    return(0);

  // save the window handle and instance in a global
  main_window_handle = hwnd;
  main_instance = hinstance;

  // perform all game console specific initialization
  Game_Init();

  // enter main event loop
  while(1) {
    if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) { 
      // test if this is a quit
      if (msg.message == WM_QUIT)
        break;
      
      // translate any accelerator keys
      TranslateMessage(&msg);
      
      // send the message to the window proc
      DispatchMessage(&msg);
    } // end if
    
    // main game processing goes here
    Game_Main();
    
  } // end while
  
  // shutdown game and release all resources
  Game_Shutdown();
  
  // return to Windows like this
  return(msg.wParam);

} // end WinMain

// WINX GAME PROGRAMMING CONSOLE FUNCTIONS
//----------------------------------------------------------------------
int Game_Init(void *parms) {
  // this function is where you do all the initialization 
  // for your game

  // seed random number generate
  srand(Start_Clock());

  // Want debugging messages defined
#ifdef DEBUG
  Open_Error_File("dragon_msg.txt");
  setbuf(fp_error, 0);
  Write_Error("Starting up application...\n\n");
  Write_Error("\t1. Opened file...\n");
#endif

  // start up DirectDraw (replace the parms as you desire)
  DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

#ifdef DEBUG
  Write_Error("\n\t2. Started DirectX...\n");
#endif
  
  // load background image
  Load_Bitmap_File(&bitmap8bit, "BACK.BMP");
  Create_Bitmap(&background_bmp,0,0,640,480);
  Load_Image_Bitmap(&background_bmp, &bitmap8bit,0,0,BITMAP_EXTRACT_MODE_ABS);
  Set_Palette(bitmap8bit.palette);
  Unload_Bitmap_File(&bitmap8bit);

#ifdef DEBUG
  Write_Error("\t3. Loaded bitmaps...\n");
#endif
  
  // initialize directinput
  DInput_Init();
  
#ifdef DEBUG
  Write_Error("\t4. Initialized DirectInput...\n");
#endif

  // acquire the keyboard only
  DInput_Init_Keyboard();

  // initilize DirectSound
  DSound_Init();
  
#ifdef DEBUG
  Write_Error("\t5. Initialized DirectSound...\n");
#endif

  // set clipping rectangle to screen extents so objects dont
  // mess up at edges
  RECT screen_rect = {0,0,screen_width,screen_height};
  lpddclipper = DDraw_Attach_Clipper(lpddsback,1,&screen_rect);
  
#ifdef DEBUG
  Write_Error("\t6. Initialized Clipping...\n");
#endif

  // Initialize game board
  Init_Board();
#ifdef DEBUG
  Write_Error("\t7. Initialized Game Board...\n");
#endif

  // hide the mouse
  ShowCursor(FALSE);

#ifdef DEBUG
  Write_Error("\t8. Finished Initializing...\n");
#endif

  // return success
  return(1);

} // end Game_Init

//----------------------------------------------------------------------
int Game_Shutdown(void *parms) {
  // this function is where you shutdown your game and
  // release all resources that you allocated
  
#ifdef DEBUG
  Write_Error("Shutting down engine...\n");
#endif

  Destroy_Board();

#ifdef DEBUG
  Write_Error("\t1. Killed BOB...\n");
#endif
  
  // shutdown directdraw last
  DDraw_Shutdown();

#ifdef DEBUG
  Write_Error("\t2. Shutdown DirectDraw...\n");
#endif

  // now directsound
  DSound_Stop_All_Sounds();
  DSound_Shutdown();

#ifdef DEBUG
  Write_Error("\t3. Shutdown DirectSound...\n");
#endif
  
  // shut down directinput
  DInput_Shutdown();

#ifdef DEBUG
  Write_Error("\t4. Shutdown DirectInput...\n");
#endif

  // close the error message file
#ifdef DEBUG
  Close_Error_File();
#endif
  
  // return success
  return(1);
  
} // end Game_Shutdown

//----------------------------------------------------------------------
int Game_Main(void *parms) {
  // this is the workhorse of your game it will be called
  // continuously in real-time this is like main() in C
  // all the calls for you game go here!
  int i;

  // start the timing clock
  Start_Clock();
  
  // clear the drawing surface
  DDraw_Fill_Surface(lpddsback, 0);
  
  // lock back buffer and copy background into it
  DDraw_Lock_Back_Surface();
  
  // draw background
  Draw_Bitmap(&background_bmp, back_buffer, back_lpitch,0);
  
  // unlock back surface
  DDraw_Unlock_Back_Surface();
  
  // read keyboard
  DInput_Read_Keyboard();

  // flip our counter for tracking the press/release state of keys
  if (! (DIKEYDOWN(keyboard_state, DIK_RIGHT)) )
    next_right = KEYDN_CONST;
  if (! (DIKEYDOWN(keyboard_state, DIK_LEFT)) )
    next_left = KEYDN_CONST;
  if (! (DIKEYDOWN(keyboard_state, DIK_DOWN)) )
    next_down = KEYDN_CONST;
  if (! (DIKEYDOWN(keyboard_state, DIK_UP)) )
    next_up = KEYDN_CONST;
  if (! (DIKEYDOWN(keyboard_state, DIK_SPACE)) )
    next_space = KEYDN_CONST;
  if (! (DIKEYDOWN(keyboard_state, DIK_P)) )
    next_p = KEYDN_CONST;

  // switch on state game is in...
  switch (curr_state) {
  case GAME_BLOCK_FALLING:
    // check the player controls
    if ( (DIKEYDOWN(keyboard_state, DIK_RIGHT)) && (keyboard_state[DIK_RIGHT] & next_right) ) {
      if ( (curr_block.col+1 < BOARD_WIDTH) && (game_matrix[curr_block.row-1][curr_block.col+1] == EMPTY_BLOCK) ) {
        curr_block.col++;
        ColPos_Curr_Block(curr_block.col);
        next_right = 0;
      }
#ifdef DEBUG
      Write_Error("\tMove right col:%d row:%d...\n", curr_block.col, curr_block.row);
#endif      

    } else if ( (DIKEYDOWN(keyboard_state, DIK_LEFT)) && (keyboard_state[DIK_LEFT] & next_left) ) {
      if ( (curr_block.col > 0) && (game_matrix[curr_block.row-1][curr_block.col-1] == EMPTY_BLOCK) ) {
        curr_block.col--;
        ColPos_Curr_Block(curr_block.col);
        next_left = 0;
      }
#ifdef DEBUG
      Write_Error("\tMove right col:%d row:%d...\n", curr_block.col, curr_block.row);
#endif      
      
    } else if ( (DIKEYDOWN(keyboard_state, DIK_UP)) && (keyboard_state[DIK_UP] & next_up) ) {
      int tmp_val;
      tmp_val = curr_block.type_list[0];
      curr_block.type_list[0] = curr_block.type_list[1];
      curr_block.type_list[1] = tmp_val;
      
      BOB* tmp_bob;
      tmp_bob = curr_block.data_list[0];
      curr_block.data_list[0] = curr_block.data_list[1];
      curr_block.data_list[1] = tmp_bob;
      
      tmp_val = curr_block.data_list[0]->y;
      curr_block.data_list[0]->y = curr_block.data_list[1]->y;
      curr_block.data_list[1]->y = tmp_val;

      // play flipping sound
      DSound_Play(sound_flip_id);

#ifdef DEBUG      
      Write_Error("\tMove swap...\n");
#endif
      
      next_up = 0;
    } else if ( (DIKEYDOWN(keyboard_state, DIK_DOWN)) && (keyboard_state[DIK_DOWN] & next_down) ) {
      Drop_Curr_Block();
      next_down = 0;
      break; // don't check falling state again
    } else if ( (DIKEYDOWN(keyboard_state, DIK_SPACE)) && (keyboard_state[DIK_SPACE] & next_space) ) {
      Drop_Curr_Block();
      next_space = 0;
      break; // don't check falling state again
    } else if ( (DIKEYDOWN(keyboard_state, DIK_P)) && (keyboard_state[DIK_P] & next_p) ) {
      curr_state = GAME_PAUSED;
      next_p = 0;
    } // end if

    // check if current has hit 'bottom'
    if (Check_Down_Curr_Block()) {
      curr_state = GAME_BLOCK_DOWN;
      // return now, so process down block
      return(1);
    }
    break;
  case GAME_BLOCK_DOWN:
    for (i=0; i<BLOCK_SIZE; i++) {
      // fix those blocks in place
      game_matrix[curr_block.row-i][curr_block.col] = curr_block.type_list[i];
#ifdef DEBUG
      Write_Error("\tsaved [%d][%d]%d\n", curr_block.row-i, curr_block.col, 
                  game_matrix[curr_block.row-i][curr_block.col]);
#endif
      bob_matrix[curr_block.row-i][curr_block.col].curr_frame = curr_block.data_list[i]->curr_frame;
    }
    curr_state = GAME_CHECK_MATRIX;

    // reset the falling bobs
    Choose_Next_Block();

    break;
  case GAME_CHECK_MATRIX: {
    // check if we have matches
    static int tot_count = 0;
    static int curr_count = 0;
    
    // remove blocks from previous call
    if (curr_count != 0) {
      // remove blocks
      Drop_Remove_Blocks();
    };

    if ((curr_count = Find_Remove_Blocks()) != 0) {
#ifdef DEBUG
      Write_Error("blocks:%d tot_blocks:%d\n", curr_count, tot_count);
#endif
      tot_count += curr_count;

      // begin animation sequence to remove blocks
      next_state = GAME_CHECK_MATRIX;
      curr_state = GAME_ANIMATE;

      // play remove block sound
      DSound_Play(sound_remove_id);

      // perform animation sequence
      Init_Anim_Remove_Blocks();
    } else {
      int old_level = curr_level;

      // finished finding matches, score points and generate another block
      curr_score += Calculate_Points(tot_count);
      curr_level = 1 + curr_score / 2000;
#ifdef DEBUG
      Write_Error("score:%d +%d blocks:%d level:%d\n", curr_score, Calculate_Points(tot_count), tot_count, curr_level);
#endif
      tot_count = curr_count = 0;

      if (old_level != curr_level)
        // play remove block sound
        DSound_Play(sound_level_id);

      curr_state = GAME_BLOCK_FALLING;
    }
  }
  break;
  case GAME_ANIMATE:
    break;
  case GAME_DEAD:
    if ( (DIKEYDOWN(keyboard_state, DIK_SPACE)) && (keyboard_state[DIK_SPACE] & next_space) ) {
      Reset_Board();
      next_space = 0;
    }
    break;
  case GAME_PAUSED:
    if ( (DIKEYDOWN(keyboard_state, DIK_P)) && (keyboard_state[DIK_P] & next_p) ) {
      curr_state = GAME_BLOCK_FALLING;
      next_p = 0;
    }
    break;
  default:
    break;
  } // end switch on state

  // draw current board/block state
  Draw_Board();

  // draw the text
  sprintf(buffer, "%d", curr_level);
  Draw_Text_GDI(buffer,490,200,RGB(255,0,0), lpddsback);
  
  sprintf(buffer,"%d", curr_score);
  Draw_Text_GDI(buffer,420,370,RGB(255,0,0), lpddsback);
  
  // flip the surfaces
  DDraw_Flip();
  
  // sync to 30 fps = 1/30sec = 33 ms
  Wait_Clock(33);

  // check of user is trying to exit
  if (KEY_DOWN(VK_ESCAPE) || keyboard_state[DIK_ESCAPE]) {
    PostMessage(main_window_handle, WM_DESTROY,0,0);

    // stop all sounds
    DSound_Stop_All_Sounds();
    
    // do a screen transition
    Screen_Transitions(SCREEN_DARKNESS,NULL,0);
  } // end if

  // return success
  return(1);
  
} // end Game_Main

//----------------------------------------------------------------------
